@@ -468,7 +468,7 @@ def plot_defect_densities(plot_params_in={}):
     if not "dens_xaxis_parameter" in plot_params_in.keys():
         plot_params_in["dens_xaxis_parameter"]="NONE"
     if not "dens_xaxis_log" in plot_params_in.keys():
-        plot_params_in["dens_xaxis_log"]=False
+        plot_params_in["dens_xaxis_log"]="NONE"
     if not "dens_x_upper_limit" in plot_params_in.keys():
         plot_params_in["dens_x_upper_limit"]="NONE"
     if not "dens_x_lower_limit" in plot_params_in.keys():
@@ -607,6 +607,12 @@ def plot_defect_densities(plot_params_in={}):
         else:
             print(f" ERROR(plot)::'dens_xaxis_parameter' cannot be defined.")
             sys.exit()
+    if plot_params_in["dens_xaxis_log"]=="NONE":
+        if plot_params_in["dens_xaxis_parameter"].find("pressure_")==0:
+            plot_params_in["dens_xaxis_log"]=True
+        else:
+            plot_params_in["dens_xaxis_log"]=False
+
     parameters_list_main=[]
     parameters_list_exclmain={}
     bool_lambda=False
