@@ -15,6 +15,9 @@ from pydecs.defects import Defects
 from pydecs.host import Host
 from pydecs.inout import plot_Edef_eFermi
 
+class LineOptError(Exception):
+    pass
+
 class DefectEqSolver:
 
     def calc_defEne_single(self,temperature_in,chempots_in,eFermi_in):
@@ -268,7 +271,7 @@ class DefectEqSolver:
                 if self.defects.check_minus_defect_energy():
                     print(" ERROR(opt_Nsites):: Densities cannot be obtained for minus defect formation energy.")
                     print("   Check output-files: *_Edef_eFermi.*  ")
-                    sys.exit()
+                    return None
                 self.update_Nsites_defective()
                 Nsites_new=self.host.get_Nsites_defective()
                 residuals_Nsites=self.calc_residual_Nsites(Nsites_previous,Nsites_new)
@@ -355,9 +358,13 @@ class DefectEqSolver:
             outdata_all.append(outdata)
         return outdata_all
 
+
     def opt_steady_line_minimization(self,func_resid,vect_grad,cp0,step_size,fout=None,tol_in=1e-7,maxiter=1000):
         fout.write(f" ------ Starting step-wize minimization with step size of {step_size} -----\n")
         residArray0=func_resid(cp0)
+        if residArray0 is None:
+            print("test-D00")
+            return (None,None)
         resid0=np.sqrt(np.square(residArray0).mean(axis=0))
         vect_ms=0.0
         for i1,v1 in enumerate(vect_grad):
@@ -372,6 +379,9 @@ class DefectEqSolver:
         i_loop=1
         while i_loop<maxiter:
             residArray1=func_resid(cp1)
+            if residArray1 is None:
+                print("test-D01")
+                return (None,None)
             resid1=np.sqrt(np.square(residArray1).mean(axis=0))
             if resid1>resid0:
                 break
@@ -392,21 +402,34 @@ class DefectEqSolver:
             for i1 in range(len(cp0)):
                 cp2.append(x_in*cp0[i1]+(1.0-x_in)*cp1[i1])
             residArray2=func_resid(cp2)
+            if residArray2 is None:
+                print("test-CHK00")
+                raise LineOptError("Minimization error!")
+                # return 0.0
             resid2=np.sqrt(np.square(residArray2).mean(axis=0))
             return resid2
-        res3=optimize.minimize_scalar(func_resid_local,bracket=(0.0,1.0),method="golden",tol=tol_in,options={'xtol':tol_in})
+        try:
+            res3=optimize.minimize_scalar(func_resid_local,bracket=(0.0,1.0),method="golden",tol=tol_in,options={'xtol':tol_in})
+        except LineOptError:
+            return(None,None)
         cp3=[]
         for i1 in range(len(cp0)):
             cp3.append(res3.x*cp0[i1]+(1.0-res3.x)*cp1[i1])
         residArray3=func_resid(cp3)
+        if residArray3 is None:
+            print("test-Z00")
+            return (None,None)
         resid3=np.sqrt(np.square(residArray3).mean(axis=0))
         print("finished: opt_steady_line_minimization")
         return(cp3,residArray3)
 
-    def opt_chempot_multi(self,func_resid,func_grad,cp0,delta_cp=0.1,fout=None,tol=1e-7,maxiter=1000):
+    def opt_chempot_multi(self,func_resid,func_grad,cp0,delta_cp,damping_rate,fout,tol,maxiter):
         cp2=copy.copy(cp0)
         delta_cp2=delta_cp
         residArray2=func_resid(cp2)
+        if residArray2 is None:
+            print("test-B02")
+            return (None,None)
         vgrad2=func_grad(residArray2)
         resid2=np.sqrt(np.square(residArray2).mean(axis=0))
         CGalgo="PR" ### PR may be best
@@ -449,7 +472,13 @@ class DefectEqSolver:
             fout.write(f" ----- New search starting ({iloop1+1}) -----\n")
             fout.write(f" ----- Gradient for the line search = {cgvect2} -----\n")
             (cp3,residArray2)=self.opt_steady_line_minimization(func_resid,cgvect2,cp2,delta_cp2,fout,tol,maxiter)
-            delta_cp2=delta_cp/(iloop1+2.0)
+            if residArray2 is None:
+                print("test-B01")
+                return (None,None)
+            if damping_rate=="DividedByNsteps":
+                delta_cp2=delta_cp/(iloop1+2.0)
+            else:
+                delta_cp2*=damping_rate
             cp2=cp3
             vgrad2=func_grad(residArray2)
             resid2=np.sqrt(np.square(residArray2).mean(axis=0))
@@ -470,6 +499,8 @@ class DefectEqSolver:
             opt_params_in["opt_Natoms_output_every_step"]=False
         if not "opt_Natoms_delta_chempot" in opt_params_in.keys():
             opt_params_in["opt_Natoms_delta_chempot"]=0.1
+        if not "opt_Natoms_delta_chempot_damping_rate" in opt_params_in.keys():
+            opt_params_in["opt_Natoms_delta_chempot_damping_rate"]="DividedByNsteps"
 
         self.reset_id_loop_Nsites()
         self.reset_id_loop_eFermi()
@@ -535,7 +566,10 @@ class DefectEqSolver:
             cp1=copy.deepcopy(chempots_in)
             for icp1,cp_in1 in enumerate(cp_in):
                 cp1[Natoms_elem_list[icp1]]=cp_in1
-            self.opt_Nsites_for_chempots(temperature_in,cp1,opt_params_in,root_outfiles)
+            out_defect_densities=self.opt_Nsites_for_chempots(temperature_in,cp1,opt_params_in,root_outfiles)
+            if out_defect_densities is None:
+                print("test-A01")
+                return None
             del_nat_array=[]
             nat_array=[]
             nat_dict=self.defects.get_Natoms_list()
@@ -555,7 +589,13 @@ class DefectEqSolver:
             return vect_grads
         (Natoms_chempot_results_list,res_delNat)=self.opt_chempot_multi(
                 func_resid_fixNatoms,func_grads_fixNatoms,
-                Natoms_chempot0_list,opt_params_in["opt_Natoms_delta_chempot"],fout1,opt_params_in["opt_Natoms_tol"])
+                Natoms_chempot0_list,opt_params_in["opt_Natoms_delta_chempot"],
+                opt_params_in["opt_Natoms_delta_chempot_damping_rate"],
+                fout1,opt_params_in["opt_Natoms_tol"],
+                opt_params_in["opt_Natoms_maxloop"])
+        if Natoms_chempot_results_list is None:
+            print("test-F01")
+            return None
         print(" [opt_Natoms] Finished")
         cp_list=copy.deepcopy(chempots_in)
         for iat1,nat1 in enumerate(Natoms_elem_list):
@@ -597,7 +637,7 @@ class DefectEqSolver:
                 for fix1 in eq_cond_in["fix_Natoms"]:
                     chempots[fix1["element"]]=fix1["chempot_init"]
             self.output_defEne_wrt_eFermi(temp,chempots,root_outfiles,plot_params_in)
-            return "NONE"
+            return None
 
         bool_fixNatoms=False
         self.eqlabel_fixNatoms="NONE"
@@ -614,6 +654,9 @@ class DefectEqSolver:
                     e1=eq1["element"]
                     eq1["chempot_init"]=self.chempots_fixNatoms[e1]
             defect_densities=self.opt_Natoms(temp,chempots,eq_cond_in[self.eqlabel_fixNatoms],solver_params_in,root_outfiles,plot_params_in)
+            if defect_densities is None:
+                print("test-E00")
+                return None
             if solver_params_in["opt_Natoms_chempot_init"]=="update":
                 self.chempots_fixNatoms={}
                 for ie1,eq1 in enumerate(eq_cond_in[self.eqlabel_fixNatoms]):
@@ -623,6 +666,6 @@ class DefectEqSolver:
                             self.chempots_fixNatoms[e1]=float(d1[-1])
         else:
             defect_densities=self.opt_Nsites_for_chempots(temp,chempots,solver_params_in,root_outfiles,plot_params_in)
-
+        
         return defect_densities
 
